/*------------------------------------------------------------------------------
#
#	MBTaskSample.r
#
#	derived from MPW 3.1's "Sample", a MultiFinder-Aware Simple Sample Application
#   - and nearly the same thing as the "TaskSample" in the neighbour folder.
#
#   Launch "MBTaskSample" first, and then "Exercise", and do A/ROSE parallel processing
#   (or rather: Program-to-program communication) even without MCP card!
#
#	Components:
#				MBTaskSample.c
#				MBTaskSample.r
#				MBTaskSample.h
#				MBTaskSample.make
#
------------------------------------------------------------------------------*/


#include "Types.r"

#include "MBTaskSample.h"



resource 'MBAR' (rMenuBar, preload) {
	{ mApple, mFile, mEdit, mSleep};
};


resource 'MENU' (mApple, preload) {
	mApple, textMenuProc,
	AllItems & ~MenuItem2,	/* Disable dashed line, enable About and DAs */
	enabled, apple,
	{
		"About MBTaskSample",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain
	}
};

resource 'MENU' (mFile, preload) {
	mFile, textMenuProc,
	AllItems,
	enabled, "File",
	{
		"Reset",
			noicon, "R", nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Quit",
			noicon, "Q", nomark, plain
	}
};

resource 'MENU' (mEdit, preload) {
	mEdit, textMenuProc,
	NoItems,				/* disable everything, program does the enabling */
	enabled, "Edit",
	 {
		"Undo",
			noicon, "Z", nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Cut",
			noicon, "X", nomark, plain;
		"Copy",
			noicon, "C", nomark, plain;
		"Paste",
			noicon, "V", nomark, plain;
		"Clear",
			noicon, nokey, nomark, plain
	}
};

resource 'MENU' (mSleep, preload) {
	mSleep, textMenuProc,
	allEnabled,
	enabled, "SleepTime",
	 {
		"no Sleep",
			noicon, nokey, nomark, plain;
		"60 Ticks",
			noicon, nokey, check, plain;
	}
};



/* this ALRT and DITL are used as an About screen */

resource 'ALRT' (rAboutAlert, purgeable) {
	{40, 20, 160, 330},
	rAboutAlert,
	{ /* array: 4 elements */
		/* [1] */
		OK, visible, silent,
		/* [2] */
		OK, visible, silent,
		/* [3] */
		OK, visible, silent,
		/* [4] */
		OK, visible, silent
	}
};

resource 'DITL' (rAboutAlert, purgeable) {
	{ /* array DITLarray: 5 elements */
		/* [1] */
		{88, 210, 108, 290},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{8, 8, 24, 330},
		StaticText {
			disabled,
			"Mandelbrot computation as an A/ROSE task"
		},
		/* [3] */
		{32, 8, 48, 330},
		StaticText {
			disabled,
			"(launch this first, then 'Exercise')"
		},
		/* [4] */
		{56, 8, 72, 136},
		StaticText {
			disabled,
			"Brought to you by:"
		},
		/* [5] */
		{80, 24, 112, 167},
		StaticText {
			disabled,
			"Joseph Maurer (EURO.DTS)"
		}
	}
};


/* this ALRT and DITL are used as an error screen */

resource 'ALRT' (rUserAlert, purgeable) {
	{40, 250, 160, 510},
	rUserAlert,
	{ /* array: 4 elements */
		/* [1] */
		OK, visible, silent,
		/* [2] */
		OK, visible, silent,
		/* [3] */
		OK, visible, silent,
		/* [4] */
		OK, visible, silent
	}
};


resource 'DITL' (rUserAlert, purgeable) {
	{ /* array DITLarray: 3 elements */
		/* [1] */
		{80, 150, 100, 230},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 60, 60, 230},
		StaticText {
			disabled,
			"^0."
		},
		/* [3] */
		{8, 8, 40, 40},
		Icon {
			disabled,
			2
		}
	}
};

resource 'STR ' (rTaskName, purgeable) {
 "MBTask"
};

resource 'STR ' (rTaskType, purgeable) {
 "MBServer"
};


resource 'STR#' (rErrStrings, purgeable) {
	{
	"You must run on 512Ke or later";
	"Application Memory Size is too small";
	"Not enough memory to run MBTaskSample";
	"Cannot create window";
	"Cannot use A/ROSE Prep";
	"Task already registered";
	"<GetMsg> failed";
	"No Server task found";
	"Problem with ICC Manager";
	"Received message with unknown mCode";
	"Received undeliverable message";
	"AROSEGetMem() failed";
	}
};

resource 'STR#' (rWndStrings, purgeable) {
	{
	" Lines computed: ";
	}
};


resource 'WIND' (rWindow, preload, purgeable) {
	{50, 260, 130, 500},
	noGrowDocProc, visible, noGoAway, 0x0, "MBTaskSample"
};



/* here is the quintessential MultiFinder friendliness device, the SIZE resource */

resource 'SIZE' (-1) {
	dontSaveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,				/* we can background; we don't currently, but our sleep value */
								/* guarantees we don't hog the Mac while we are in the background */
	multiFinderAware,			/* this says we do our own activate/deactivate; don't fake us out */
	backgroundAndForeground,	/* this is definitely not a background-only application! */
	dontGetFrontClicks,			/* change this is if you want "do first click" behavior like the Finder */
	ignoreChildDiedEvents,		/* essentially, I'm not a debugger (sub-launching) */
	not32BitCompatible,			/* this app should not be run in 32-bit address space */
	reserved,
	reserved,
	reserved,
	reserved,
	reserved,
	reserved,
	reserved,
	kPrefSize * 1024,
	kMinSize * 1024	
};
