/*------------------------------------------------------------------------------
#
#	MBosmain.c
#
#	Example of an A/ROSE task, to be linked with "MBosmain.c.o"
#  and downloaded to a MCP card.
#
#  Registers itself with 
#  object_name  "MBTask"  and
#  type_name    "MBServer".
#
#  The "Exercise" application in this same folder will then find
#  this task on a MCP card, and use it for parallel processing
#  (if the "MCP" checkbox is checked).
#  Look at the source code Exercise.c for more details about the 
#  distributed Mandelbrot computation.
#
#	Components:
#				MBosmain.c
#				iter16.a
#				MBTask.c
#
------------------------------------------------------------------------------*/
/* -------------- MPW: select the following lines + <ENTER> ------
Asm iter16.a
C MBosmain.c -r
C MBTask.c   -r
link -t 'DMRP' -c 'RWM ' 
	MBosmain.c.o 
	MBTask.c.o 
	iter16.a.o 
	::OS.o 
	::osglue.o 
	-o start
	
::Download start
---------------------------------------------------------------- */


// the following definitions come from the A/ROSE include file "os.h"

typedef long tid_type;

struct ST_Registers
{
	long	D_Registers [8];		// D0 - D7 
	long	A_Registers [8];		// A0 - A7  Note: A7 not used
	long	PC;						// Program Counter
};

struct ST_PB
{
	char	*CodeSegment;			// memory region on card for code
	char	*DataSegment;			// memory region on card for global data
	char	*StartParmSegment;		// memory region on card for start parameters
	struct	ST_Registers InitRegs;	// initial register set for starting task
	long	stack;					// initial stack size (in bytes)
	long	heap;					// initial heap size (in bytes)
	short	return_code;			// error code if task not started (Tid = 0)
	unsigned char priority;			// priority of task (in bytes)
	tid_type	ParentTID;			// TID of Parent on Network/Host
};

typedef struct ST_PB ST_PB;



// the following declarations come from arose.h

typedef unsigned long 	OSlong;
typedef unsigned short 	OSshort;
typedef unsigned char 	OSchar;

//	gCommon - A/ROSE common data area

struct	gCommon		//	Located at 0x400 on the MCP card
{
	OSshort	gRelease;			//	release and version number
	OSchar	gProcessor;			//	Type of processor		
	OSchar	gCardRun;			//	Set zero by download, non-zero by O/S
	OSshort	gBoardID;			//	Board id of card		
	OSlong	gTID;				//	Current Task ID			
	OSchar	gDebugOn;			//	non-zero if debugger running		
	OSchar	gMajorFlag;			//	major processing flag	
	OSlong	gInitA5;			//	Initial A5				
	// ................
	// many, many other fields contained in this structure: look up in arose.h !
	// ................
};

typedef struct gCommon gCommon;

#define		cMaxMsg		500			//	Maximum number of messages
#define		cOSStack	4096		//	size of OS Stack

// the following two lines come from siop.h

#define	TICKS_PS	19				//	19 ticks per second	
#define	TICK_MIN_MAJ 8				//   8 minor ticks per major tick


// prototypes

void	osinit ();
void	osstart ();
void	name_server ();
void	ICCM ();
extern	tid_type		GetTID();
extern	struct gCommon	*GetgCommon();
extern	tid_type		StartTask(struct ST_PB *);

void	MBTask ();

void StartNameServer(ST_PB *pb);	// the Name Manager
void StartICCManager(ST_PB *pb);	// you guess it!
void StartMBTask(ST_PB *pb);		// and our Mandelbrot task

pascal void illegal ()
		extern	0x4afc;


main ()
{
	struct ST_PB stpb;	// Start parameter block

	// Init OS with cMaxMsg messages and cStackOS stack

	osinit (cMaxMsg, cOSStack);
	
	StartNameServer(&stpb);	// the Name Manager
	StartICCManager(&stpb);	// you guess it!
	StartMBTask(&stpb);		// and our sample task
	
	osstart (TICK_MIN_MAJ, TICKS_PS);	// Start operating system

	illegal ();	  // should never get here!

}



void StartNameServer(ST_PB *pb)
// priority 31, 4k stack, 0 heap
{
	pb -> CodeSegment = 0;
	pb -> DataSegment = 0;
	pb -> StartParmSegment = 0;
	pb -> stack = 4096;
	pb -> heap = 0;
	pb -> priority = 31;
	pb -> InitRegs.PC = name_server;
	pb -> InitRegs.A_Registers [5] = GetgCommon() -> gInitA5;
	pb -> ParentTID = GetTID();
	
	if (StartTask (pb) == 0)
		illegal ();
}

void StartICCManager(ST_PB *pb)
// priority 31, 128-byte stack, 0 heap
{
	pb -> CodeSegment = 0;
	pb -> DataSegment = 0;
	pb -> StartParmSegment = 0;
	pb -> stack = 128;
	pb -> heap = 0;
	pb -> priority = 31;
	pb -> InitRegs.PC = ICCM;
	pb -> InitRegs.A_Registers [5] = GetgCommon() -> gInitA5;
	pb -> ParentTID = GetTID();
	
	if (StartTask (pb) == 0)
		illegal ();
}


void StartMBTask(ST_PB *pb)
// priority 30, 4k stack, 0 heap
{
	pb -> CodeSegment = 0;
	pb -> DataSegment = 0;
	pb -> StartParmSegment = 0;
	pb -> InitRegs.A_Registers [5] = GetgCommon() -> gInitA5;
	pb -> ParentTID = GetTID();
	pb -> stack = 4096;
	pb -> heap = 0;
	pb -> priority = 10;
	pb -> InitRegs.PC = MBTask;	// entry point of Mandelbrot task

	if (StartTask (pb) == 0)	// if the task does not get started
		illegal ();				// go debugging
}

